"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.selectBuildableWorkspaces = selectBuildableWorkspaces;
exports.selectWorkspaces = selectWorkspaces;
var _workspaceResolver = require("@modular-scripts/workspace-resolver");
var _packageTypes = require("./packageTypes");
var _getAllWorkspaces = require("./getAllWorkspaces");
var _getChangedWorkspaces = require("./getChangedWorkspaces");
var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
/**
 * Select all target packages in workspaces in random order, optionally including changed, ancestors and descendant packages
 *
 * This method will not throw, even if there are circular dependencies in the graph
 *
 * @param  {SelectOptions} options The target options to configure selection
 * @return {Promise<string[]>} A distinct list of selected package names
 */

async function selectWorkspaces(options) {
  return [...new Set((await computeWorkspaceSelection(options)).packageScope)];
}

/**
 * Select buildable target packages in workspaces in build order, optionally including changed, ancestors and descendant packages
 *
 * Please note that the build order algorithm can't calculate build order if there's a cycle in the dependency graph,
 * so circular dependencies will throw unless they only involve packages that don't get built (i.e. "source" `modular.type`s).
 * Please also note that circular dependencies can always be fixed by creating an additional package that contains the common parts,
 * and that they are considered a code smell + a source of many issues (e.g. https://nodejs.org/api/modules.html#modules_cycles)
 * and they make your code fragile towards refactoring, so please don't introduce them in your monorepository.
 *
 * @param  {SelectBuildableOptions} options The target options to configure selection
 * @return {Promise<string[]>} A distinct list of selected buildable package names, in build order
 */

async function selectBuildableWorkspaces(options) {
  const {
    ancestors,
    descendants,
    dangerouslyIgnoreCircularDependencies
  } = options;
  const {
    packageScope,
    isBuildable,
    allWorkspacesMap,
    descendantsSet,
    ancestorsSet,
    targetsToBuild
  } = await computeWorkspaceSelection(options);
  if (!dangerouslyIgnoreCircularDependencies) {
    // Here we're using traverseWorkspaceRelations to warn us if there's at least one cyclical dependency in the graph
    // Since the dependency graph can be disconnected, it makes only sense to calculate cycles in the context
    // of a subset of packages we want to build (aka the package scope)
    void (0, _workspaceResolver.traverseWorkspaceRelations)(packageScope, allWorkspacesMap);
  }
  const buildablePackageScope = [...new Set(packageScope)].filter(isBuildable);
  // Since buildOrder is true, we want to select packages in build order. The build order algorithm gives up if there's a cycle in the dependency graph
  // (since it can't know what to build first if A depends on B but B depends on A), so we can allow cycles only if they involve packages that are not built
  // (i.e. "source" `modular.type`s). Please note that dependency cycles can always be fixed by creating an additional package that contains the common parts,
  // and that circular dependencies are a source of many additional issues.

  // The package graph is reduced to a graph where all vertices are buildable.
  // To do that, we need to first filter the vertices, then filter the edges that might be pointing to a non-buildable vertex
  const buildableWorkspacesMap = Object.fromEntries(Object.entries(allWorkspacesMap).filter(([name]) => isBuildable(name)).map(([name, workspace]) => {
    return [name, {
      ...workspace,
      workspaceDependencies: workspace.workspaceDependencies.filter(isBuildable)
    }];
  }));

  // traverseWorkspaceRelations will walk the given graph (in this case, the graph of all buildable packages) and generate the whole ordering needed to build a package
  // which means that it will possibly expand the scope (it just generates the dependency order starting from a dependency subset but taking into account the whole dependency graph)
  // this means that we can generate an order from any strict subset of packages, but we need to filter out all the unwanted packages later.
  const targetEntriesWithOrder = [...(0, _workspaceResolver.traverseWorkspaceRelations)(buildablePackageScope, buildableWorkspacesMap).entries()];
  return targetEntriesWithOrder.sort((a, b) => b[1] - a[1]).map(([packageName]) => packageName)
  // Filter out descendants and ancestors if we don't explicitly need them.
  // This allows us to get the correct dependency order even if we restrict the scope (for example, by explicit user input).
  .filter(packageName => descendants && descendantsSet.has(packageName) || ancestors && ancestorsSet.has(packageName) || targetsToBuild.includes(packageName));
}

/**
 * Common data structures to calculate package selection
 */
async function computeWorkspaceSelection({
  targets,
  changed,
  ancestors,
  descendants,
  compareBranch
}) {
  const [allWorkspacePackages, allWorkspacesMap] = await (0, _getAllWorkspaces.getAllWorkspaces)((0, _getModularRoot.default)());
  let changedTargets = [];
  if (changed) {
    const [, buildTargetMap] = await (0, _getChangedWorkspaces.getChangedWorkspacesContent)(compareBranch);
    changedTargets = Object.keys(buildTargetMap);
  }
  const targetsToBuild = [...new Set(targets.concat(changedTargets))];

  // We want to remove all the non-buildable packages from the package scope. Create a filter predicate that looks up to the package map
  const isBuildable = name => {
    var _allWorkspacePackages, _allWorkspacePackages2;
    const type = (_allWorkspacePackages = allWorkspacePackages.get(name)) === null || _allWorkspacePackages === void 0 ? void 0 : (_allWorkspacePackages2 = _allWorkspacePackages.modular) === null || _allWorkspacePackages2 === void 0 ? void 0 : _allWorkspacePackages2.type;
    return type && (0, _packageTypes.isBuildableModularType)(type);
  };
  let ancestorsSet = new Set();
  let descendantsSet = new Set();
  let packageScope = [];
  if (targetsToBuild.length) {
    // Calculate the package scope, i.e. the deduped array of all the packages we need to generate an ordering graph for.
    // We need to remember ancestor and descendant sets in order to select them later.

    packageScope = targetsToBuild;
    if (descendants) {
      descendantsSet = (0, _workspaceResolver.computeDescendantSet)(targetsToBuild, allWorkspacesMap);
      packageScope = packageScope.concat([...descendantsSet]);
    }
    if (ancestors) {
      ancestorsSet = (0, _workspaceResolver.computeAncestorSet)(targetsToBuild, allWorkspacesMap);
      packageScope = packageScope.concat([...ancestorsSet]);
    }
  }
  return {
    packageScope,
    isBuildable,
    allWorkspacesMap,
    ancestorsSet,
    descendantsSet,
    targetsToBuild
  };
}
//# sourceMappingURL=selectWorkspaces.js.map